from __future__ import annotations

import os
from textwrap import dedent

import flake8

from pytest_flake8_path import Flake8Result

# Prevent Flake8 from running in dev mode because it returns some problems
os.environ.pop("PYTHONDEVMODE", None)


def test_windows_path_normalization():
    result = Flake8Result(
        out=(
            ".\\example.py:1:2: E221 multiple spaces before operator\n"
            + "Flake8 is here!\n"
        ),
        err="",
        exit_code=0,
    )
    assert result.out_lines == [
        "./example.py:1:2: E221 multiple spaces before operator",
        "Flake8 is here!",
    ]


def test_simple_run(flake8_path):
    (flake8_path / "example.py").write_text("x  = 1\n")

    result = flake8_path.run_flake8()

    assert result.out_lines == [
        "./example.py:1:2: E221 multiple spaces before operator"
    ]
    assert result.err_lines == []
    assert result.exit_code == 1


def test_multi_line(flake8_path):
    (flake8_path / "example.py").write_text(
        dedent(
            """\
            x  = 1
            y  = 2
            """
        )
    )

    result = flake8_path.run_flake8()

    assert result.out_lines == [
        "./example.py:1:2: E221 multiple spaces before operator",
        "./example.py:2:2: E221 multiple spaces before operator",
    ]
    assert result.err_lines == []
    assert result.exit_code == 1


def test_with_setup_cfg(flake8_path):
    (flake8_path / "setup.cfg").write_text(
        dedent(
            """\
            [flake8]
            ignore = E221
            """
        )
    )
    (flake8_path / "example.py").write_text("x  = 1\n")

    result = flake8_path.run_flake8()

    assert result.out_lines == []
    assert result.err_lines == []
    assert result.exit_code == 0


def test_error_output(flake8_path):
    (flake8_path / "setup.cfg").write_text(
        dedent(
            """
            [flake8]
            extend-select = MC1

            [flake8:local-plugins]
            extension =
                MC1 = example:MyChecker1
            paths = .
            """
        )
    )
    (flake8_path / "example.py").write_text(
        dedent(
            """
            import warnings

            warnings.warn("This is a warning!", UserWarning)


            class MyChecker1:
                name = "MyChecker1"
                version = "1"

                def __init__(self, tree):
                    self.tree = tree

                def run(self):
                    if False:
                        yield
            """
        )
    )

    result = flake8_path.run_flake8()

    assert "This is a warning!" in result.err
    assert result.err_lines[0].endswith("UserWarning: This is a warning!")
    assert result.err_lines[1] == '  warnings.warn("This is a warning!", UserWarning)'


def test_extra_args(flake8_path):
    (flake8_path / "example.py").write_text("x  = 1\n")

    result = flake8_path.run_flake8(["--ignore", "E221"])

    assert result.out_lines == []
    assert result.err_lines == []
    assert result.exit_code == 0


def test_extra_args_version(flake8_path):
    result = flake8_path.run_flake8(extra_args=["--version"])
    assert result.out.startswith(flake8.__version__ + " ")


def test_separate_tmp_path(flake8_path, tmp_path):
    (flake8_path / "example.py").write_text("x  = 1\n")
    assert not (tmp_path / "example.py").exists()



from __future__ import annotations

import subprocess
import sys
from collections.abc import Generator
from pathlib import Path
from typing import TYPE_CHECKING

import pytest
from _pytest.tmpdir import TempPathFactory


class Flake8Result:
    def __init__(self, out: str, err: str, exit_code: int) -> None:
        self.out = out
        self.out_lines: list[str] = []
        for line in out.strip().splitlines():
            if ":" in line:
                filename, rest = line.split(":", 1)
                norm_filename = filename.replace("\\", "/")
                self.out_lines.append(f"{norm_filename}:{rest}")
            else:
                self.out_lines.append(line)
        self.err = err
        self.err_lines = err.strip().splitlines()
        self.exit_code = exit_code


if TYPE_CHECKING:
    BasePathType = Path
else:
    # Have to extend concrete type, which is determined in Path.__new__
    BasePathType = type(Path())


class Flake8Path(BasePathType):
    def run_flake8(self, extra_args: list[str] | None = None) -> Flake8Result:
        args = [
            sys.executable,
            "-m",
            "flake8",
            "--jobs",
            "1",
            "--config",
            "setup.cfg",
            ".",
        ]
        if extra_args:
            args.extend(extra_args)

        process = subprocess.run(
            args=args,
            cwd=str(self),
            capture_output=True,
            text=True,
        )
        return Flake8Result(
            out=process.stdout,
            err=process.stderr,
            exit_code=process.returncode,
        )


@pytest.fixture
def flake8_path(tmp_path_factory: TempPathFactory) -> Generator[Flake8Path]:
    path = Flake8Path(tmp_path_factory.mktemp("flake8_path"))
    (path / "setup.cfg").write_text("[flake8]\n")
    yield path
